import isObject from 'lodash/isObject';
import { ProviderResolver, ProviderDetector, BatchRequest, HttpProvider, WebsocketProvider, IpcProvider } from 'conflux-web-providers';
import { MethodProxy } from 'conflux-web-core-method';
import { toChecksumAddress } from 'conflux-web-utils';

function _classCallCheck(instance, Constructor) {
  if (!(instance instanceof Constructor)) {
    throw new TypeError("Cannot call a class as a function");
  }
}

function _defineProperties(target, props) {
  for (var i = 0; i < props.length; i++) {
    var descriptor = props[i];
    descriptor.enumerable = descriptor.enumerable || false;
    descriptor.configurable = true;
    if ("value" in descriptor) descriptor.writable = true;
    Object.defineProperty(target, descriptor.key, descriptor);
  }
}

function _createClass(Constructor, protoProps, staticProps) {
  if (protoProps) _defineProperties(Constructor.prototype, protoProps);
  if (staticProps) _defineProperties(Constructor, staticProps);
  return Constructor;
}

var AbstractConfluxWebModule = function () {
  function AbstractConfluxWebModule(provider) {
    var _this = this;
    var options = arguments.length > 1 && arguments[1] !== undefined ? arguments[1] : {};
    var methodFactory = arguments.length > 2 && arguments[2] !== undefined ? arguments[2] : null;
    var nodeNet = arguments.length > 3 && arguments[3] !== undefined ? arguments[3] : null;
    _classCallCheck(this, AbstractConfluxWebModule);
    this.providerResolver = new ProviderResolver();
    this.givenProvider = ProviderDetector.detect();
    this._currentProvider = this.providerResolver.resolve(provider, nodeNet);
    this._defaultAccount = options.defaultAccount ? toChecksumAddress(options.defaultAccount) : undefined;
    this._defaultEpoch = options.defaultEpoch || 'latest_state';
    this._transactionBlockTimeout = options.transactionBlockTimeout || 50;
    this._transactionConfirmationBlocks = options.transactionConfirmationBlocks || 24;
    this._transactionPollingTimeout = options.transactionPollingTimeout || 750;
    this._defaultGasPrice = options.defaultGasPrice;
    this._defaultGas = options.defaultGas;
    this.BatchRequest = function () {
      return new BatchRequest(_this);
    };
    if (methodFactory) {
      return new MethodProxy(this, methodFactory);
    }
  }
  _createClass(AbstractConfluxWebModule, [{
    key: "setProvider",
    value: function setProvider(provider, net) {
      if (!this.isSameProvider(provider)) {
        var resolvedProvider = this.providerResolver.resolve(provider, net);
        this.clearSubscriptions();
        this._currentProvider = resolvedProvider;
        return true;
      }
      return false;
    }
  }, {
    key: "isSameProvider",
    value: function isSameProvider(provider) {
      if (isObject(provider)) {
        if (this.currentProvider.constructor.name === provider.constructor.name) {
          return this.currentProvider.host === provider.host;
        }
        return false;
      }
      return this.currentProvider.host === provider;
    }
  }, {
    key: "clearSubscriptions",
    value: function clearSubscriptions(unsubscribeMethod) {
      if (typeof this.currentProvider.clearSubscriptions !== 'undefined' && this.currentProvider.subscriptions.length > 0) {
        return this.currentProvider.clearSubscriptions(unsubscribeMethod);
      }
      return Promise.resolve(true);
    }
  }, {
    key: "defaultEpoch",
    get: function get() {
      return this._defaultEpoch;
    }
    ,
    set: function set(value) {
      this._defaultEpoch = value;
    }
  }, {
    key: "transactionBlockTimeout",
    get: function get() {
      return this._transactionBlockTimeout;
    }
    ,
    set: function set(value) {
      this._transactionBlockTimeout = value;
    }
  }, {
    key: "transactionConfirmationBlocks",
    get: function get() {
      return this._transactionConfirmationBlocks;
    }
    ,
    set: function set(value) {
      this._transactionConfirmationBlocks = value;
    }
  }, {
    key: "transactionPollingTimeout",
    get: function get() {
      return this._transactionPollingTimeout;
    }
    ,
    set: function set(value) {
      this._transactionPollingTimeout = value;
    }
  }, {
    key: "defaultGasPrice",
    get: function get() {
      return this._defaultGasPrice;
    }
    ,
    set: function set(value) {
      this._defaultGasPrice = value;
    }
  }, {
    key: "defaultGas",
    get: function get() {
      return this._defaultGas;
    }
    ,
    set: function set(value) {
      this._defaultGas = value;
    }
  }, {
    key: "defaultAccount",
    get: function get() {
      return this._defaultAccount;
    }
    ,
    set: function set(value) {
      this._defaultAccount = toChecksumAddress(value);
    }
  }, {
    key: "currentProvider",
    get: function get() {
      return this._currentProvider;
    }
    ,
    set: function set(value) {
      throw new Error('The property currentProvider is read-only!');
    }
  }], [{
    key: "providers",
    get: function get() {
      return {
        HttpProvider: HttpProvider,
        WebsocketProvider: WebsocketProvider,
        IpcProvider: IpcProvider
      };
    }
  }]);
  return AbstractConfluxWebModule;
}();

export { AbstractConfluxWebModule };
